@@ -31,6 +31,7 @@
 
 #include <QObject>
 #include <QVariant>
+#include <QVector>
 #include <QJsonValue>
 #include <QJsonObject>
 #include <QJsonArray>
@@ -53,10 +54,18 @@ public:
         emit messageReceived(message, this);
     }
 
-public slots:
-    void sendMessage(const QJsonObject &/*message*/) Q_DECL_OVERRIDE
+    QVector<QJsonObject> messagesSent() const
     {
+        return mMessagesSent;
     }
+
+public slots:
+    void sendMessage(const QJsonObject &message) Q_DECL_OVERRIDE
+    {
+        mMessagesSent.push_back(message);
+    }
+private:
+    QVector<QJsonObject> mMessagesSent;
 };
 
 class TestObject : public QObject
@@ -258,20 +267,21 @@ public:
     explicit TestWebChannel(QObject *parent = 0);
     virtual ~TestWebChannel();
 
+public slots:
     int readInt() const;
-    Q_INVOKABLE void setInt(int i);
+    void setInt(int i);
     bool readBool() const;
-    Q_INVOKABLE void setBool(bool b);
+    void setBool(bool b);
     double readDouble() const;
-    Q_INVOKABLE void setDouble(double d);
+    void setDouble(double d);
     QVariant readVariant() const;
-    Q_INVOKABLE void setVariant(const QVariant &v);
+    void setVariant(const QVariant &v);
     QJsonValue readJsonValue() const;
-    Q_INVOKABLE void setJsonValue(const QJsonValue &v);
+    void setJsonValue(const QJsonValue &v);
     QJsonObject readJsonObject() const;
-    Q_INVOKABLE void setJsonObject(const QJsonObject &v);
+    void setJsonObject(const QJsonObject &v);
     QJsonArray readJsonArray() const;
-    Q_INVOKABLE void setJsonArray(const QJsonArray &v);
+    void setJsonArray(const QJsonArray &v);
 
 signals:
     void lastIntChanged();
@@ -285,6 +295,7 @@ signals:
 private slots:
     void testRegisterObjects();
     void testDeregisterObjects();
+    void testDeregisterObjectAtStart();
     void testInfoForObject();
     void testInvokeMethodConversion();
     void testSetPropertyConversion();
@@ -294,6 +305,8 @@ private slots:
     void testPassWrappedObjectBack();
     void testInfiniteRecursion();
     void testAsyncObject();
+    void testDeletionDuringMethodInvocation_data();
+    void testDeletionDuringMethodInvocation();
 
     void benchClassInfo();
     void benchInitializeClients();
